#!/usr/bin/env python
# encoding: utf-8
"""
check_prerequisites.py

Created by Rod Docking on 2011-04-18.
Copyright (c) 2011 Canada's Michael Smith Genome Sciences Centre. All rights reserved.
"""

import sys
import os
import subprocess
import re

def check_prerequisites():
    """
    Check whether all requirements are installed and are up-to-date
    """
    
    # Check for Python version (though it will probably fail well before here if it's wrong!)
    assert(sys.version_info[0] == 2)
    assert(sys.version_info[1] in [6,7])
            
    # Check that ABYSS version is correct
    args = ['ABYSS', '--version']
    try:
        proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stdout=subprocess.PIPE)
        version_string = proc.stdout.readlines()[0].rstrip()
    except OSError, e:
        print "\n\nCouldn't find ABySS - please ensure that it's on your PATH\n"
        raise

    abyss_major, abyss_minor, abyss_micro = [int(i) for i in version_string.split()[2].split('.')]
    if not ((abyss_major == 1) and (abyss_minor >= 3) and (abyss_micro >= 0)):
        print "Anchor is tied to a specific ABySS version (at least 1.3.0)"
        print "Found:", version_string
        sys.exit()

    # TODO - give DAssembler a proper version string
    args = ['DAssembler', '--help']
    try:
        proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stdout=subprocess.PIPE)
        version_string = proc.stdout.readlines()[0].rstrip()
    except OSError, e:
        print "\n\nCouldn't find DAssembler - please ensure that it's on your PATH\n"
        raise

    assert(version_string == "Usage: DAssembler [OPTION]... [READS]")
    
    # Check that samtools exists
    args = ['samtools']
    try:
        proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stderr=subprocess.PIPE)
    except OSError, e:
        print "\n\nCouldn't find samtools - please ensure that it's on your PATH\n"
        raise
    
    # And that the version is recent enough
    for line in proc.stderr:
        c = line.split()
        if len(c) < 3:
            continue        
        if "Version:" in c[0]:
            fields = [int(x) for x in re.split('[\.-]', c[1])]
            major, minor, micro = fields[:3]
            assert(minor >= 1)
            assert(micro >= 18)

    # Check for blat
    args = ['blat']
    try:
        proc = subprocess.Popen(args, shell=False, cwd=os.getcwd(), stdout=subprocess.PIPE)
    except OSError, e:
        print "\n\nCouldn't find blat - please ensure that it's on your PATH\n"
        raise
    
    return


if __name__ == '__main__':
    check_prerequisites()

